@@ -46,6 +46,11 @@ GType gnome_vfs_open_mode_get_type (void);
 GType gnome_vfs_seek_position_get_type (void);
 #define GNOME_VFS_TYPE_VFS_SEEK_POSITION (gnome_vfs_seek_position_get_type())
 
+/* enumerations from "gnome-vfs-mime-utils.h" */
+
+GType gnome_vfs_mime_equivalence_get_type (void);
+#define GNOME_VFS_TYPE_VFS_MIME_EQUIVALENCE (gnome_vfs_mime_equivalence_get_type())
+
 /* enumerations from "gnome-vfs-mime-handlers.h" */
 
 GType gnome_vfs_mime_action_type_get_type (void);
