@@ -184,6 +184,8 @@ typedef enum {
 } GnomeVFSFilePermissions;
 
 
+#define GNOME_VFS_TYPE_FILE_INFO  (gnome_vfs_file_info_get_type ())
+
 /**
  * GnomeVFSFileInfo:
  * 
@@ -406,6 +408,8 @@ typedef struct {
 
 
 
+GType             gnome_vfs_file_info_get_type      (void);
+
 GnomeVFSFileInfo *gnome_vfs_file_info_new           (void);
 void              gnome_vfs_file_info_unref         (GnomeVFSFileInfo       *info);
 void              gnome_vfs_file_info_ref           (GnomeVFSFileInfo       *info);
