@@ -30,7 +30,6 @@
 #include <gconf/gconf-client.h>
 #include "gnome-vfs-i18n.h"
 #include "gnome-vfs-volume-monitor-private.h"
-#include "gnome-vfs-volume-monitor-daemon.h"
 #include "gnome-vfs-volume.h"
 #include "gnome-vfs-utils.h"
 #include "gnome-vfs-drive.h"
@@ -153,23 +152,27 @@ force_probe (void)
 	GnomeVFSClient *client;
 	GNOME_VFS_Daemon daemon;
 	CORBA_Environment  ev;
+	GnomeVFSDaemonForceProbeCallback callback;
 	
 	volume_monitor = gnome_vfs_get_volume_monitor ();
 
 	if (gnome_vfs_get_is_daemon ()) {
-		gnome_vfs_volume_monitor_daemon_force_probe (GNOME_VFS_VOLUME_MONITOR_DAEMON (volume_monitor));
+		callback = _gnome_vfs_get_daemon_force_probe_callback();
+		(*callback) (GNOME_VFS_VOLUME_MONITOR (volume_monitor));
 	} else {
 		client = _gnome_vfs_get_client ();
 		daemon = _gnome_vfs_client_get_daemon (client);
-		
-		CORBA_exception_init (&ev);
-		GNOME_VFS_Daemon_forceProbe (daemon,
-					     BONOBO_OBJREF (client),
-					     &ev);
-		if (BONOBO_EX (&ev)) {
-			CORBA_exception_free (&ev);
+
+		if (daemon != CORBA_OBJECT_NIL) {
+			CORBA_exception_init (&ev);
+			GNOME_VFS_Daemon_forceProbe (daemon,
+						     BONOBO_OBJREF (client),
+						     &ev);
+			if (BONOBO_EX (&ev)) {
+				CORBA_exception_free (&ev);
+			}
+			CORBA_Object_release (daemon, NULL);
 		}
-		CORBA_Object_release (daemon, NULL);
 	}
 }
 
@@ -288,6 +291,9 @@ mount_unmount_thread (void *arg)
 				info->succeeded = FALSE;
 				info->error_message = g_strdup (_("Unable to eject media"));
 				info->detailed_error_message = g_strdup (standard_error);
+			} else {
+				/* If the eject succeed then ignore the previous unmount error (if any) */
+				info->succeeded = TRUE;
 			}
 		} else {
 			/* Spawn failure */
@@ -388,15 +394,25 @@ emit_pre_unmount (GnomeVFSVolume *volume)
 		client = _gnome_vfs_get_client ();
 		daemon = _gnome_vfs_client_get_daemon (client);
 		
-		CORBA_exception_init (&ev);
-		 GNOME_VFS_Daemon_emitPreUnmountVolume (daemon,
-							BONOBO_OBJREF (client),
-							gnome_vfs_volume_get_id (volume),
-							&ev);
-		 if (BONOBO_EX (&ev)) {
-			 CORBA_exception_free (&ev);
-		 }
-		 CORBA_Object_release (daemon, NULL);
+		if (daemon != CORBA_OBJECT_NIL) {
+			CORBA_exception_init (&ev);
+			GNOME_VFS_Daemon_emitPreUnmountVolume (daemon,
+							       BONOBO_OBJREF (client),
+							       gnome_vfs_volume_get_id (volume),
+							       &ev);
+			if (BONOBO_EX (&ev)) {
+				CORBA_exception_free (&ev);
+			}
+			CORBA_Object_release (daemon, NULL);
+		}
+		/* Do a synchronous pre_unmount for this client too, avoiding
+		 * races at least in this process
+		 */
+		gnome_vfs_volume_monitor_emit_pre_unmount (volume_monitor,
+							   volume);
+		/* sleep for a while to get other apps to release their
+		 * hold on the device */
+		g_usleep (0.5*G_USEC_PER_SEC);
 	}
 }
 
@@ -551,28 +567,21 @@ gnome_vfs_drive_unmount (GnomeVFSDrive  *drive,
 			 GnomeVFSVolumeOpCallback  callback,
 			 gpointer                   user_data)
 {
-	char *mount_path, *device_path, *uri;
-	GnomeVFSVolume *volume;
+	GList *vol_list;
+	GList *current_vol;
 
+	vol_list = gnome_vfs_drive_get_mounted_volumes (drive);
 
-	volume = gnome_vfs_drive_get_mounted_volume (drive);
+	for (current_vol = vol_list; current_vol != NULL; current_vol = current_vol->next) {
+		GnomeVFSVolume *vol;
+		vol = GNOME_VFS_VOLUME (current_vol->data);
 
-	if (volume != NULL) {
-		emit_pre_unmount (volume);
+		gnome_vfs_volume_unmount (vol,
+					  callback,
+					  user_data);
 	}
 
-	uri = gnome_vfs_drive_get_activation_uri (drive);
-	mount_path = gnome_vfs_get_local_path_from_uri (uri);
-	g_free (uri);
-	device_path = gnome_vfs_drive_get_device_path (drive);
-	mount_unmount_operation (mount_path,
-				 device_path,
-				 (volume == NULL) ? GNOME_VFS_DEVICE_TYPE_UNKNOWN:gnome_vfs_volume_get_device_type (volume),
-				 FALSE, FALSE,
-				 callback, user_data);
-	g_free (mount_path);
-	g_free (device_path);
-	gnome_vfs_volume_unref (volume);
+	gnome_vfs_drive_volume_list_free (vol_list);
 }
 
 void
@@ -580,28 +589,31 @@ gnome_vfs_drive_eject (GnomeVFSDrive  *drive,
 		       GnomeVFSVolumeOpCallback  callback,
 		       gpointer                   user_data)
 {
-	char *mount_path, *device_path, *uri;
-	GnomeVFSVolume *volume;
+	GList *vol_list;
+	GList * current_vol;
 
+	vol_list = gnome_vfs_drive_get_mounted_volumes (drive);
 
-	volume = gnome_vfs_drive_get_mounted_volume (drive);
+	for (current_vol = vol_list; current_vol != NULL; current_vol = current_vol->next) {
+		GnomeVFSVolume *vol;
+		vol = GNOME_VFS_VOLUME (current_vol->data);
+
+		/* Check to see if this is the last volume */
+		/* If not simply unmount it */
+		/* If so the eject the media along with the unmount */
+		if (current_vol != NULL) { 
+			gnome_vfs_volume_unmount (vol,
+						  callback,
+						  user_data);
+		} else { 
+			gnome_vfs_volume_eject (vol,
+						callback,
+						user_data);
+		}
 
-	if (volume != NULL) {
-		emit_pre_unmount (volume);
 	}
 
-	uri = gnome_vfs_drive_get_activation_uri (drive);
-	mount_path = gnome_vfs_get_local_path_from_uri (uri);
-	g_free (uri);
-	device_path = gnome_vfs_drive_get_device_path (drive);
-	mount_unmount_operation (mount_path,
-				 device_path,
-				 (volume == NULL) ? GNOME_VFS_DEVICE_TYPE_UNKNOWN:gnome_vfs_volume_get_device_type (volume),
-				 FALSE, TRUE,
-				 callback, user_data);
-	g_free (mount_path);
-	g_free (device_path);
-	gnome_vfs_volume_unref (volume);
+	gnome_vfs_drive_volume_list_free (vol_list);
 }
 
 
