@@ -136,6 +136,7 @@ gnome_vfs_volume_finalize (GObject *object)
 	g_free (priv->display_name);
 	g_free (priv->icon);
 	g_free (priv->gconf_id);
+	g_free (priv->hal_udi);
 	g_free (priv);
 	volume->priv = NULL;
 	
@@ -205,6 +206,12 @@ gnome_vfs_volume_get_activation_uri (GnomeVFSVolume *volume)
 	return g_strdup (volume->priv->activation_uri);
 }
 
+char *
+gnome_vfs_volume_get_hal_udi (GnomeVFSVolume *volume)
+{
+	return g_strdup (volume->priv->hal_udi);
+}
+
 char *
 gnome_vfs_volume_get_filesystem_type (GnomeVFSVolume *volume)
 {
@@ -335,8 +342,8 @@ decode_corba_string_or_null (CORBA_char *str, gboolean empty_is_null)
 }
 
 void
-_gnome_vfs_volume_to_corba (GnomeVFSVolume *volume,
-			    GNOME_VFS_Volume *corba_volume)
+gnome_vfs_volume_to_corba (GnomeVFSVolume *volume,
+			   GNOME_VFS_Volume *corba_volume)
 {
 	GnomeVFSDrive *drive;
 
@@ -357,6 +364,7 @@ _gnome_vfs_volume_to_corba (GnomeVFSVolume *volume,
 	corba_volume->display_name = corba_string_or_null_dup (volume->priv->display_name);
 	corba_volume->icon = corba_string_or_null_dup (volume->priv->icon);
 	corba_volume->gconf_id = corba_string_or_null_dup (volume->priv->gconf_id);
+	corba_volume->hal_udi = corba_string_or_null_dup (volume->priv->hal_udi);
 	
 	corba_volume->is_user_visible = volume->priv->is_user_visible;
 	corba_volume->is_read_only = volume->priv->is_read_only;
@@ -379,7 +387,7 @@ _gnome_vfs_volume_from_corba (const GNOME_VFS_Volume *corba_volume,
 		volume->priv->drive = gnome_vfs_volume_monitor_get_drive_by_id (volume_monitor,
 										corba_volume->drive);
 		if (volume->priv->drive != NULL) {
-			_gnome_vfs_drive_set_mounted_volume (volume->priv->drive, volume);
+			_gnome_vfs_drive_add_mounted_volume (volume->priv->drive, volume);
 			/* The drive reference is weak */
 			gnome_vfs_drive_unref (volume->priv->drive);
 		}
@@ -392,6 +400,7 @@ _gnome_vfs_volume_from_corba (const GNOME_VFS_Volume *corba_volume,
 	volume->priv->display_name = decode_corba_string_or_null (corba_volume->display_name, TRUE);
 	volume->priv->icon = decode_corba_string_or_null (corba_volume->icon, TRUE);
 	volume->priv->gconf_id = decode_corba_string_or_null (corba_volume->gconf_id, TRUE);
+	volume->priv->hal_udi = decode_corba_string_or_null (corba_volume->hal_udi, TRUE);
 	
 	volume->priv->is_user_visible = corba_volume->is_user_visible;
 	volume->priv->is_read_only = corba_volume->is_read_only;
